import type { Settings as LayoutSettings } from '@ant-design/pro-layout'
import { notification } from 'antd'
import type { RequestConfig, RunTimeLayoutConfig } from '@umijs/max'
import { history } from '@umijs/max'
import RightContent from '@/components/RightContent'
import { currentUser as queryCurrentUser } from './services/user/api'
import consts from './utils/consts'
import defaultSettings from '../config/defaultSettings'
import { isNullOrUnDef, isObject } from './utils/is'
const loginPath = '/user/login'

/**
 * @see  https://umijs.org/zh-CN/plugins/plugin-initial-state
 * */
export async function getInitialState(): Promise<{
  settings?: Partial<LayoutSettings>
  currentUser?: API.CurrentUser
  menuList?: API.MenuListItem[]
  fetchUserInfo?: () => Promise<API.CurrentUser | undefined>
}> {
  // eslint-disable-next-line react-hooks/rules-of-hooks
  // 获取用户信息
  const fetchUserInfo = async () => {
    const currentUser = await queryCurrentUser()
    return currentUser
  }

  // 如果是登录页面，不执行
  if (history.location.pathname !== loginPath) {
    try {
      const { data: currentUser } = await fetchUserInfo()
      return {
        fetchUserInfo,
        menuList: currentUser.menus,
        currentUser: currentUser.staff,
        settings: {}
      }
    } catch (error) {
      history.push(loginPath)
    }
    return undefined
  }
  return {
    settings: {}
  }
}

const authHeaderInterceptor = options => {
  const token = JSON.parse(window.localStorage.getItem('TOKEN_ID'))
  // 如果是登录页面，不执行
  if (token) {
    // 在白名单里的请求放过
    if (consts.TOKEN_WHITE_LIST.includes(options.url)) {
      return options
    }
    options.headers[consts.TOKEN_HEADER] = 'bearer ' + token
  }
  return options
}

const errorHandler = (error: any, opts: any) => {
  if (opts?.skipErrorHandler) return

  const errorInfo = error.info
  if (errorInfo) {
    const { errorMessage = '请求失败', errorCode } = errorInfo
    if (consts.TOKEN_INVALID_CODE.includes(errorCode) && window.location.pathname !== consts.loginPath) {
      notification.error({
        message: '用户信息过期',
        description: '请重新登录'
      })
      history.replace({
        pathname: consts.loginPath,
        search: createSearchParams({
          redirect: window.location.pathname
        }).toString()
      })
      return
    }
    notification.error({
      message: '请求失败',
      description: errorMessage
    })
  } else if (error.response) {
    // Axios 的错误
    // 请求成功发出且服务器也响应了状态码，但状态代码超出了 2xx 的范围
    notification.error({
      description: `状态码为${error.status}, 请联系管理员进行处理`,
      message: '请求异常'
    })
  } else if (error.request) {
    // 请求已经成功发起，但没有收到响应
    // 或请求根本没有发送出去
    notification.error({
      description: '您的网络请求已经发起，但没有收到响应',
      message: '响应超时'
    })
  } else {
    notification.error({
      description: '您的网络发生异常，无法连接服务器',
      message: '网络异常'
    })
  }
}

export const request: RequestConfig = {
  baseURL: consts.PREFIX_URL,
  errorConfig: {
    errorHandler
  },
  responseInterceptors: [
    async response => {
      const { data, code: errorCode, msg } = response.data
      if (isNullOrUnDef(errorCode) || errorCode !== consts.RET_CODE.SUCCESS) {
        const errorMessage = isObject(msg) ? JSON.stringify(msg) : msg
        const error: any = new Error(errorMessage)
        error.name = 'BizError'
        error.info = { errorCode, errorMessage, data }
        throw error
      }
      return Promise.resolve(response)
    }
  ],
  requestInterceptors: [authHeaderInterceptor]
}

// ProLayout 支持的api https://procomponents.ant.design/components/layout
export const layout: RunTimeLayoutConfig = ({ initialState }) => {
  return {
    rightContentRender: () => <RightContent />,
    disableContentMargin: false,
    waterMarkProps: initialState.currentUser && {
      fontColor: 'rgba(0,0,0,.1)',
      offsetTop: 200,
      content: `${initialState.currentUser?.username}  ${initialState.currentUser?.telephone.slice(-4)}`
    },
    onPageChange: () => {
      const { location } = history
      // 如果没有登录，重定向到 login
      if (!initialState?.currentUser && location.pathname !== loginPath) {
        history.push(loginPath)
      }
    },
    menuHeaderRender: undefined,
    // 自定义 403 页面
    // unAccessible: <div>unAccessible</div>,
    ...defaultSettings
  }
}
